VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ObjectProperties"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit


Private m_os As ObjectStore
Private m_objNode As MSXML.IXMLDOMNode


Public Function Load(ByVal propertyName As String) As Variant
    Dim propertyValue As Variant
    
    LoadProperty m_objNode, propertyName, propertyValue
    
    If IsObject(propertyValue) Then
        Set Load = propertyValue
    Else
        Load = propertyValue
    End If
End Function


Public Sub Store(ByVal propertyName As String, ByVal value As Variant)
    StoreProperty m_objNode, propertyName, value
End Sub


'---------------------------


Friend Sub Constructor(ByVal os As ObjectStore, ByVal objNode As MSXML.IXMLDOMNode)
    Set m_os = os
    Set m_objNode = objNode
End Sub


'---------------------------


Private Sub StoreProperty(ByVal parentNode As MSXML.IXMLDOMNode, ByVal propertyName As String, ByVal value As Variant)
    If InStr(propertyName, ".") = 0 Then
        Dim p As MSXML.IXMLDOMNode, nodeName As String, a As MSXML.IXMLDOMAttribute
        
        Select Case TypeName(value)
            Case "String"
                nodeName = "string": GoSub StoreSimpleTypeValue
                
                Set a = parentNode.ownerDocument.createAttribute("xml:space")
                a.nodeValue = "preserve"
                p.Attributes.setNamedItem a

            Case "Byte", "Integer", "Long"
                nodeName = "int": GoSub StoreSimpleTypeValue
            Case "Single", "Double", "Currency", "Decimal"
                value = Replace(value, ",", ".")
                    ' the float value has to be stored with a decimal point regardless
                    ' of the locale!
                nodeName = "float": GoSub StoreSimpleTypeValue
            Case "Boolean"
                value = Abs(value)
                nodeName = "boolean": GoSub StoreSimpleTypeValue
            Case "Date"
                value = Format(value, "YYYY-MM-DDTHH:MM:SS")
                nodeName = "dateTime": GoSub StoreSimpleTypeValue

            Case Else
                If IsObject(value) Then
                    value = m_os.SerializeObject(value)
                    nodeName = "objectRef": GoSub StoreSimpleTypeValue
                Else
                    If InStr(TypeName(value), "()") > 0 Then
                        Dim u%
                        On Error Resume Next
                        u = UBound(value, 2)
                        If Err <> 0 Then
                            On Error GoTo 0
                            StoreArray parentNode, propertyName, value
                        Else
                            On Error GoTo 0
                            Err.Raise errNoMultiDimArraysSupported, "StoreProperty", "Property '" & propertyName & "': No multidimensional arrays supported!"
                        End If
                    Else
                        Err.Raise errUnknownType, "StoreProperty", "Property '" & propertyName & "' is of unknown type '" & TypeName(value) & "'!"
                    End If
                End If
        End Select
        
    Else
        StoreStructMember parentNode, propertyName, value
    End If
    
    Exit Sub
    
StoreSimpleTypeValue:
    Set p = parentNode.ownerDocument.createNode(NODE_ELEMENT, nodeName, parentNode.namespaceURI)
        Set a = parentNode.ownerDocument.createAttribute("name")
        a.nodeValue = propertyName
        p.Attributes.setNamedItem a
        
    p.appendChild parentNode.ownerDocument.createTextNode(value)
    
    parentNode.appendChild p
    Return
End Sub


Private Sub LoadProperty(ByVal parentNode As MSXML.IXMLDOMNode, ByVal propertyName As String, ByRef propertyValue As Variant)
    If InStr(propertyName, ".") = 0 Then
        Dim pNode As MSXML.IXMLDOMNode
        Set pNode = parentNode.selectSingleNode("*[@name='" & propertyName & "']")
    
        Select Case pNode.nodeName
            Case "dateTime"
                ' MSXML does not support the dateTime.iso8601 datatype so we have to
                ' convert the data ourselves
                Dim dtText As String
                dtText = pNode.Text
                propertyValue = DateSerial(CInt(Left$(dtText, 4)), CInt(Mid$(dtText, 6, 2)), CInt(Mid$(dtText, 9, 2))) + _
                       TimeSerial(CInt(Mid$(dtText, 12, 2)), CInt(Mid$(dtText, 15, 2)), CInt(Mid$(dtText, 18, 2)))
            
            Case "array"
                propertyValue = LoadArray(pNode)
                
            Case "objectRef"
                Set propertyValue = m_os.DeserializeObject(pNode.Text)
                
            Case Else
                propertyValue = pNode.nodeTypedValue
        End Select
    Else
        LoadStructMember parentNode, propertyName, propertyValue
    End If
End Sub


Private Function LoadArray(ByVal parentNode As MSXML.IXMLDOMNode) As Variant
    Dim lb%, ub%, i%
    lb = parentNode.Attributes.getNamedItem("lowerBound").nodeValue
    ub = parentNode.Attributes.getNamedItem("upperBound").nodeValue
    
    ReDim a(lb To ub)
    For i = lb To ub
        LoadProperty parentNode, i, a(i)
    Next
    
    LoadArray = a
End Function


Private Sub StoreArray(ByVal parentNode As MSXML.IXMLDOMNode, ByVal propertyName As String, ByVal value As Variant)
    Dim i%, arrayNode As MSXML.IXMLDOMNode
    
    Dim a As MSXML.IXMLDOMAttribute
    Set arrayNode = parentNode.ownerDocument.createNode(NODE_ELEMENT, "array", parentNode.namespaceURI)
    Set a = parentNode.ownerDocument.createAttribute("name")
        a.nodeValue = propertyName
        arrayNode.Attributes.setNamedItem a
    Set a = parentNode.ownerDocument.createAttribute("lowerBound")
        a.nodeValue = LBound(value)
        arrayNode.Attributes.setNamedItem a
    Set a = parentNode.ownerDocument.createAttribute("upperBound")
        a.nodeValue = UBound(value)
        arrayNode.Attributes.setNamedItem a
    
    parentNode.appendChild arrayNode

    For i = LBound(value) To UBound(value)
        StoreProperty arrayNode, i, value(i)
    Next
End Sub


Private Sub StoreStructMember(ByVal parentNode As MSXML.IXMLDOMNode, ByVal propertyName As String, ByVal structMemberValue As Variant)
    Dim i%, structName As String, structNode As MSXML.IXMLDOMNode
    
    i = InStr(propertyName, ".")
    structName = Left$(propertyName, i - 1)
    propertyName = Mid$(propertyName, i + 1)
    
    Set structNode = parentNode.selectSingleNode("struct[@name='" & structName & "']")
    If structNode Is Nothing Then
        Dim a As MSXML.IXMLDOMAttribute
        Set structNode = parentNode.ownerDocument.createNode(NODE_ELEMENT, "struct", parentNode.namespaceURI)
        Set a = parentNode.ownerDocument.createAttribute("name")
            a.nodeValue = structName
            structNode.Attributes.setNamedItem a
        parentNode.appendChild structNode
    End If
    
    StoreProperty structNode, propertyName, structMemberValue
End Sub


Private Sub LoadStructMember(ByVal parentNode As MSXML.IXMLDOMNode, ByVal propertyName As String, ByRef structMemberValue As Variant)
    Dim i%, structName As String, structNode As MSXML.IXMLDOMNode
    
    i = InStr(propertyName, ".")
    structName = Left$(propertyName, i - 1)
    propertyName = Mid$(propertyName, i + 1)
    
    Set structNode = parentNode.selectSingleNode("struct[@name='" & structName & "']")
    
    LoadProperty structNode, propertyName, structMemberValue
End Sub

